module bwi_encode_mod
  use  base_struct_mod
  use  base_ioutil_mod
  implicit none

  complex, dimension(:,:,:), allocatable, private :: orec
  complex, dimension(:)    , allocatable, private :: osou
  complex, dimension(:,:,:), allocatable, private :: csou
  complex, dimension(:,:,:), allocatable, private :: crec

  type(axis)                            , private :: agx, agy, aws, asx, asy
  type(axis)                            , private :: arx, ary, apsx, apsy

contains


  integer function bwi_encode_init(osou_, orec_, csou_, crec_, offset) result(stat)
    character(len=*) :: osou_, orec_, csou_, crec_
    logical          :: offset
    real             :: sx_min, sx_max, sy_min, sy_max
    real             :: gx_min, gx_max, gy_min, gy_max
    real             :: rx_min, rx_max, ry_min, ry_max

    call auxinout(osou_)
    call auxinout(orec_)
    call auxinout(csou_)
    call auxinout(crec_)

    call hinaxis(orec_, agx, 1)
    call hinaxis(orec_, agy, 2)
    call hinaxis(orec_, aws, 3)
    call hinaxis(orec_, asx, 4)
    call hinaxis(orec_, asy, 5)

    sx_min = asx%o; sx_max = asx%o+(asx%n-1)*asx%d
    sy_min = asy%o; sy_max = asy%o+(asy%n-1)*asy%d 
    gx_min = agx%o; gx_max = agx%o+(agx%n-1)*agx%d
    gy_min = agy%o; gy_max = agy%o+(agy%n-1)*agy%d
    
    rx_min = gx_min; rx_max = gx_max
    ry_min = gy_min; ry_max = gy_max

    if (offset) then
       gx_min = gx_min + sx_min; gx_max = gx_max + sx_max
       gy_min = gy_min + sy_min; gy_max = gy_max + sy_max
    end if

    rx_min = min(sx_min, gx_min); rx_max = max(sx_max, gx_max)
    ry_min = min(sy_min, gy_min); ry_max = max(sy_max, gy_max)

    arx%o = rx_min; arx%d = min(asx%d, agx%d); arx%n = 1.5+(rx_max-arx%o)/arx%d
    ary%o = ry_min; ary%d = min(asy%d, agy%d); ary%n = 1.5+(ry_max-ary%o)/ary%d

    apsx%n = 1; apsx%d = 1.; apsx%o = 0.
    apsy%n = 1; apsy%d = 1.; apsy%o = 0.

    call houaxis(csou_, arx , 1)
    call houaxis(csou_, ary , 2)
    call houaxis(csou_, aws , 3)
    call houaxis(csou_, apsx, 4)
    call houaxis(csou_, apsy, 5)
    call auxputch("esize", "i", 8, csou_)

    call houaxis(crec_, arx , 1)
    call houaxis(crec_, ary , 2)
    call houaxis(crec_, aws , 3)
    call houaxis(crec_, apsx, 4)
    call houaxis(crec_, apsy, 5)
    call auxputch("esize", "i", 8, crec_)

    allocate(osou(aws%n))
    allocate(orec(agx%n,agy%n,aws%n))
    allocate(csou(arx%n,ary%n,aws%n))
    allocate(crec(arx%n,ary%n,aws%n))

    stat = 0
  end function

  integer function bwi_encode_clos(osou_, orec_, csou_, crec_) result(stat)
    character(len=*) :: osou_, orec_, csou_, crec_
    deallocate(osou, orec, csou, crec)
    call auxclose(osou_)
    call auxclose(orec_)
    call auxclose(csou_)
    call auxclose(crec_) 
    stat = 0 
  end function

  integer function bwi_encode_oper(osou_, orec_, csou_, crec_, offset) result(stat)
    character(len=*) :: osou_, orec_, csou_, crec_
    logical          :: offset

    real             :: gx_min, gx_max, gy_min, gy_max 
    real             :: sx, sy, rd, ws
    integer          :: isx, isy, ipsx, ipsy, iws
    integer          :: jgxb, jgxe, jgyb, jgye, jsx, jsy
    real             :: PI

    PI = 3.14159265359 

    call random_seed() 

    osou = 0.
    orec = 0.
    csou = 0.
    crec = 0.

    call sseek(osou_, 0, 0)
    call sseek(orec_, 0, 0)
    call sseek(csou_, 0, 0)
    call sseek(crec_, 0, 0) 
    call sep_read(osou, osou_)
    do isy=1, asy%n; sy = asy%o+(isy-1)*asy%d; jsy = 1.5+(sy-ary%o)/ary%d
    do isx=1, asx%n; sx = asx%o+(isx-1)*asx%d; jsx = 1.5+(sx-arx%o)/arx%d
       gx_min = agx%o; gx_max = agx%o+(agx%n-1)*agx%d
       gy_min = agy%o; gy_max = agy%o+(agy%n-1)*agy%d
       if (offset) then
          gx_min = gx_min + sx; gx_max = gx_max + sx
          gy_min = gy_min + sy; gy_max = gy_max + sy
       end if
       jgxb = 1.5+(gx_min-arx%o)/arx%d; jgxe = 1.5+(gx_max-arx%o)/arx%d
       jgyb = 1.5+(gy_min-ary%o)/ary%d; jgye = 1.5+(gy_max-ary%o)/ary%d
       call sep_read(orec, orec_)

!       call random_number(rd)
!       if (rd < 0.5) then
!          rd = -1
!       else
!          rd = +1
!       end if

       do iws=1, aws%n
          call random_number(rd)
          rd = rd*2*PI
          ws = aws%o + (iws-1)*aws%d
          csou(jsx,jsy,iws) = csou(jsx,jsy,iws) + osou(iws)*cmplx(cos(rd),sin(rd))
          crec(jgxb:jgxe,jgyb:jgye,iws) = crec(jgxb:jgxe,jgyb:jgye,iws) + orec(:,:,iws)*cmplx(cos(rd),sin(rd))
       end do
    end do
    end do
    call sep_write(csou, csou_)
    call sep_write(crec, crec_)
    stat = 0
  end function

end module
